package com.loafle.overflow.probe.container.server;


import com.fasterxml.jackson.databind.ObjectMapper;
import com.loafle.overflow.probe.container.rpc.invoker.Invoker;
import com.loafle.overflow.probe.container.rpc.protocol.json.ServerRequest;
import com.loafle.overflow.probe.container.rpc.protocol.json.ServerResponse;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.io.IOException;

@Component
@Qualifier("containerJSONHandler")
@ChannelHandler.Sharable
public class ContainerJSONHandler extends SimpleChannelInboundHandler<String> {
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private Invoker invoker;

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, String s) throws Exception {
        ServerRequest req = objectMapper.readValue(s, ServerRequest.class);
        ServerResponse res = null;
        try {
            Object result = invoker.invoke(req.getMethod(), req.getParams());
            if (null != req.getId()) {
                res = new ServerResponse();
                res.setId(req.getId());
                res.setResult(result);
            }

        } catch (Exception e) {
            if (null != req.getId()) {
                res = new ServerResponse();
                res.setId(req.getId());
                res.setError(e.getMessage());
            }
        }

        if (null != res) {
            String json = objectMapper.writeValueAsString(res);
            System.out.println(json);
            ctx.writeAndFlush(json);
        }
    }

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {

    }


    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        ctx.close();
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        super.exceptionCaught(ctx, cause);
    }
}
